@@ -67,7 +67,11 @@ static bool nets_parse(const vscf_data_t* nets_cfg, dclists_t* dclists, const ch
 
     for(unsigned i = 0; i < input_nnets; i++) {
         // convert 192.0.2.0/24 -> anysin_t w/ mask in port field
-        char* net_str = strdup(vscf_hash_get_key_byindex(nets_cfg, i, NULL));
+        unsigned net_str_len = 0;
+        const char* net_str_cfg = vscf_hash_get_key_byindex(nets_cfg, i, &net_str_len);
+        char net_str[net_str_len + 1];
+        memcpy(net_str, net_str_cfg, net_str_len + 1);
+
         char* mask_str = strchr(net_str, '/');
         if(!mask_str) {
             log_err("plugin_geoip: map '%s': nets entry '%s' does not parse as addr/mask", map_name, net_str);
@@ -113,8 +117,6 @@ static bool nets_parse(const vscf_data_t* nets_cfg, dclists_t* dclists, const ch
             memcpy(&ipv6[12], &tempsin.sin.sin_addr.s_addr, 4);
         }
 
-        free(net_str);
-
         // get dclist integer from rhs
         const vscf_data_t* dc_cfg = vscf_hash_get_data_byindex(nets_cfg, i);
         const unsigned dclist = dclists_find_or_add_vscf(dclists, dc_cfg, map_name, false);
