@@ -88,7 +88,7 @@ static void mon_interval_cb(struct ev_loop* loop, struct ev_timer* t, const int
     dmn_assert(md->sock == -1);
     dmn_assert(!ev_is_active(md->read_watcher));
     dmn_assert(!ev_is_active(md->write_watcher));
-    dmn_assert(!ev_is_active(md->timeout_watcher));
+    dmn_assert(!ev_is_active(md->timeout_watcher) && !ev_is_pending(md->timeout_watcher));
 
     log_debug("plugin_http_status: Starting state poll of %s", md->smgr->desc);
 
@@ -154,7 +154,7 @@ static void mon_write_cb(struct ev_loop* loop, struct ev_io* io, const int reven
     dmn_assert(md->hstate == HTTP_STATE_WRITING);
     dmn_assert(!ev_is_active(md->read_watcher));
     dmn_assert(ev_is_active(md->write_watcher));
-    dmn_assert(ev_is_active(md->timeout_watcher));
+    dmn_assert(ev_is_active(md->timeout_watcher) || ev_is_pending(md->timeout_watcher));
     dmn_assert(md->sock > -1);
 
     int sock = md->sock;
